
/*
** hooks.c
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2.1 of the License, or
** (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

#include "alarm_type.h"

#define OMSG_ALARM_TYPE_ACCEPTED (0x1f00)
#define OMSG_SKIP_ONCE_SELECTED  (0x1f01)

extern rexx_uint alarm_type_dispatcher_id;

extern rexx_uint eprid_alarm;
extern rexx_uint confid_days;
extern rexx_uint confid_skip;

extern   const nokstr_t str_skip_once_selected;

/* this will be hooked into the Alarm Clock Setting dispatcher */
unsigned hook_alarm_clock(unsigned msg)
{
 switch(msg)
 {
  /* here we can initialize our patch */
  case MSG_D_INIT:
  {
   /* we set it to 3, to mark, that our alarm type dispatcher wasn't called yet */
   app.format = 3;
   break;
  }

  /* this message comes when a correct time was entered and the dialog was accepted;
     its only arg is a value telling how the time should be converted before use
     (0 - none, 1 - time is 12h/am, 2 - time is 12h/pm) */
  case 0x6ba:
  {
   app.format=msg_argv[0]; /* time format has to be saved for later */
   /* we start our handler, it will handle asking for type of alarm and the weekdays */
   send_message(MSG_START_HANDLER|MSG_1P,alarm_type_dispatcher_id);
   /* we have to discard this message to prevent alarm clock handler from setting the
      alarm yet (and finishing) */
   msg=MSG_NOP;
   break;
  }

  /* this message comes when our handler is finished, arg 0 tells us if it succeeded */
  case MSG_D_QUIT_NOW:
  {
   if( app.format != 3 ) /* set by our handler, we have to check this since we may return from time setting dispatcher */
   {
    /* if OK (succeeded), we have to send 6ba to the underlying alarm clock dispatcher,
       because this would be catched by the above 'case', we send different message and
       replace it when it comes (and we do some other things there too) */
    if(msg_argv[0]==QNS_OK) {
       own_set_config( confid_days, app.days );
       own_set_config( confid_skip, false );
       send_message(OMSG_ALARM_TYPE_ACCEPTED|MSG_1P,app.format);
    } else {
     /* if failed, Back was chosen; if the format was 24h, we have to display time dialog, if
        it was 12h, we have to return to am/pm menu */
     if(app.format==0) send_message(0x6bb); /* 6bb causes the time dialog to be displayed again, if the user pressed Back */
     else menu_reopen(); /* reopen the am/pm menu */
    }
    msg=MSG_R_STAY; /* without this dispatcher will end */
   }
   break;
  }

  /* this message comes when our handler returns with success, arg 0 is the format so we
     can simply replace the msg to get 6ba/format */
  case OMSG_ALARM_TYPE_ACCEPTED:
  {
   msg=0x6ba; /* replacing, the alarm will be set */
   break;
  }

   /* skip once was selected (while alarm is already set) */
   case OMSG_SKIP_ONCE_SELECTED:
      own_set_config( confid_skip, true );
      display_msg( DMI_DONE, &str_skip_once_selected );
      msg = MSG_NOP;
      break;
 }
 return msg;
}

/* return today's alarm status
   0x0000            = Repeat on, no alarm today
   0x0001            = Repeat off
   0x0002...0x0080   = Repeat on, alarm today
   0x0100            = Repeat off, skip once
   0x0200...0x8000   = Repeat on, alarm today, skip once
*/
static unsigned int alarm_today(void) {
   unsigned int skip, days;

   skip = own_get_config(confid_skip) * 8;
   days = own_get_config(confid_days);
   if( days & 1 ) {
      return ( days & ( 2 << own_extract_weekday(get_timestamp()) ) ) << skip;
   }
   return 1 << skip;
}

/* this will be hooked into task 5 (UI) */
ULONG hook_task5(ULONG msg)
{
 switch(msg)
 {
  /* this message comes when the time expires and the alarm should be generated, we are hooked just before it */
  case MSG_ALARM:
  {
   /* if skip once or shouldn't alarm today */
   if( ( alarm_today() & 0x00FF ) == 0x0000 ) {

     /* this will set the alarm for the next day (see below) */
     send_message(MSG_ALARM_STOP);
     /* this checks if the phone is in POWER_ON/RTC_CLOCK state meaning it was powered up
        by the alarm, if yes, we have to turn the phone off silently */
     if(get_phone_state()==PHST_POWER_ON_RTC) send_message(0x795);
     msg=MSG_NOP; /* beep won't be generated */
   }
   break;
  }
  /* this message comes when user selects to stop the beeping (or stop snooze),
     we use this place to restart the alarm for next day */
  case MSG_ALARM_STOP:
  {
   unsigned int status = alarm_today();

   if( status != 0x0001 ) {    /* if we have repeated alarm set or skip once */
      struct datetime alarm;
      
      /* clear skip once if should have alarmed today */
      if( status >= 0x0100 ) own_set_config( confid_skip, false );

      /* we simply calculate a new timestamp without care if we should alarm tomorrow,
         we have to do it like this because alarm can only be set within 24 hours,
         if we shouldn't alarm tomorrow, this will be catched by MSG_ALARM handler (above) */
      memcpy(&alarm,&datetime_zero,sizeof(struct datetime));
      eeprom_read(&alarm.hours,eprid_alarm,0); /* here the phone stores the time struct for alarm clock */
      send_message(MSG_SET_ALARM|MSG_1P,calc_timestamp(&datetime_zero,&alarm));
   }
   break;
  }
 }
 return msg;
}

